use crate::{prelude::*, router::NewRouterError};
use prelude::*;
use thiserror::Error;

mod prelude {
    pub use clap::{Args, Parser, Subcommand};
}

/// Web application for managing lyrics and live displays
#[derive(Parser)]
#[command(version, about, long_about = None)]
#[command(propagate_version = true)]
pub struct Cli {
    #[command(subcommand)]
    command: Commands,
}

#[derive(Subcommand)]
enum Commands {
    /// Run the web application server
    Run(Run),
}

/// Doc comment
#[derive(Args)]
struct Run {
    /// Whether or not to watch certain resource files for changes and reload accordingly
    #[arg(short, long, default_value = None)]
    pub watch: bool,

    /// The address to bind to - you almost certainly want to use :: or 0.0.0.0 instead of the default
    #[arg(short = 'H', long, default_value = "::1")]
    pub host: String,

    /// The port to bind to
    #[arg(short, long, default_value = "3000")]
    pub port: u16,
}

#[derive(Error, Debug)]
pub enum RunError {
    #[error("router error: {0}")]
    Router(#[from] NewRouterError),

    #[error("io error: {0}")]
    Io(#[from] std::io::Error),
}

impl Run {
    pub async fn run(&self) -> Result<(), RunError> {
        let (router, _watchers) = crate::router::router(self.watch).await?;
        Ok(
            crate::webserver::webserver(router, self.watch, Some(&self.host), Some(self.port))
                .await?,
        )
    }
}

pub fn cli() -> Cli {
    Cli::parse()
}

#[derive(Error, Debug)]
pub enum ExecError {
    #[error("run error: {0}")]
    Run(#[from] RunError),
}

impl Cli {
    pub async fn exec(self) -> Result<(), ExecError> {
        match self.command {
            Commands::Run(args) => Ok(args.run().await?),
        }
    }
}
